@@ -8,13 +8,13 @@
 #include <osmocom/gprs/gprs_ns2.h>
 #include <osmocom/gprs/gprs_bssgp.h>
 
-#include <osmocom/sgsn/gprs_sgsn.h>
+#include <osmocom/sgsn/auth.h>
 #include <osmocom/sgsn/gtp_mme.h>
 #include <osmocom/gsm/oap_client.h>
 #include <osmocom/gsupclient/gsup_client.h>
 #include <osmocom/sgsn/common.h>
 
-#include "../../bscconfig.h"
+#include "../../config.h"
 
 #if BUILD_IU
 #include <osmocom/ranap/iu_client.h>
@@ -25,6 +25,8 @@
 
 struct hostent;
 
+#define SGSN_ERROR_CAUSE_NONE (-1)
+
 enum sgsn_auth_policy {
 	SGSN_AUTH_POLICY_OPEN,
 	SGSN_AUTH_POLICY_CLOSED,
@@ -75,8 +77,10 @@ struct sgsn_config {
 	/* misc */
 	struct gprs_ns2_inst *nsi;
 
+	char *crypt_cipher_plugin_path;
 	enum sgsn_auth_policy auth_policy;
-	uint8_t cipher_support_mask;
+	uint8_t gea_encryption_mask;
+	uint8_t uea_encryption_mask;
 	struct llist_head imsi_acl;
 
 	struct sockaddr_in gsup_server_addr;
@@ -92,6 +96,7 @@ struct sgsn_config {
 
 	/* Timer defintions */
 	struct osmo_tdef *T_defs;
+	struct osmo_tdef *T_defs_gtp;
 
 	int dynamic_lookup;
 
@@ -149,10 +154,22 @@ struct sgsn_instance {
 
 	struct rate_ctr_group *rate_ctrs;
 
+	struct llist_head apn_list; /* list of struct sgsn_apn_ctx */
+	struct llist_head ggsn_list; /* list of struct sgsn_ggsn_ctx */
 	struct llist_head mme_list; /* list of struct sgsn_mme_ctx */
+	struct llist_head mm_list; /* list of struct sgsn_mm_ctx */
+	struct llist_head pdp_list; /* list of struct sgsn_pdp_ctx */
+
+	struct ctrl_handle *ctrlh;
 };
 
 extern struct sgsn_instance *sgsn;
+extern void *tall_sgsn_ctx;
+
+/*
+ * ctrl interface related work (sgsn_ctrl.c)
+ */
+int sgsn_ctrl_cmds_install(void);
 
 /* sgsn_vty.c */
 
@@ -161,40 +178,13 @@ int sgsn_parse_config(const char *config_file);
 char *sgsn_gtp_ntoa(struct ul16_t *ul);
 
 /* sgsn.c */
-
-/* Main input function for Gb proxy */
-int sgsn_rcvmsg(struct msgb *msg, struct gprs_ns2_vc *nsvc, uint16_t ns_bvci);
-
-/* sgsn_libgtp.c */
-struct sgsn_pdp_ctx *sgsn_create_pdp_ctx(struct sgsn_ggsn_ctx *ggsn,
-					 struct sgsn_mm_ctx *mmctx,
-					 uint16_t nsapi,
-					 struct tlv_parsed *tp);
-int sgsn_delete_pdp_ctx(struct sgsn_pdp_ctx *pctx);
-void sgsn_pdp_upd_gtp_u(struct sgsn_pdp_ctx *pdp, void *addr, size_t alen);
-void sgsn_ggsn_echo_req(struct sgsn_ggsn_ctx *ggc);
-int send_act_pdp_cont_acc(struct sgsn_pdp_ctx *pctx);
-int sgsn_mme_ran_info_req(struct sgsn_mme_ctx *mme, const struct bssgp_ran_information_pdu *pdu);
-
-/* gprs_sndcp.c */
-
-/* Entry point for the SNSM-ACTIVATE.indication */
-int sndcp_sm_activate_ind(struct gprs_llc_lle *lle, uint8_t nsapi);
-/* Entry point for the SNSM-DEACTIVATE.indication */
-int sndcp_sm_deactivate_ind(struct gprs_llc_lle *lle, uint8_t nsapi);
-/* Called by SNDCP when it has received/re-assembled a N-PDU */
-int sgsn_rx_sndcp_ud_ind(struct gprs_ra_id *ra_id, int32_t tlli, uint8_t nsapi,
-			 struct msgb *msg, uint32_t npdu_len, uint8_t *npdu);
-int sndcp_unitdata_req(struct msgb *msg, struct gprs_llc_lle *lle, uint8_t nsapi,
-			void *mmcontext);
-int sndcp_llunitdata_ind(struct msgb *msg, struct gprs_llc_lle *lle,
-			 uint8_t *hdr, uint16_t len);
-
-
+struct sgsn_instance *sgsn_instance_alloc(void *talloc_ctx);
+int sgsn_inst_init(struct sgsn_instance *sgsn);
 /*
  * CDR related functionality
  */
 int sgsn_cdr_init(struct sgsn_instance *sgsn);
+void sgsn_cdr_release(struct sgsn_instance *sgsn);
 
 
 /*
