@@ -417,7 +417,7 @@ static int create_pdp_conf(struct pdp_t *pdp, void *cbp, int cause)
 	}
 
 	/* Check for cause value if it was really successful */
-	if (cause != GTPCAUSE_ACC_REQ) {
+	if (!gtp_cause_successful(cause)) {
 		reject_cause = cause_map(gtp2sm_cause_map, cause,
 					 GSM_CAUSE_ACT_REJ_GGSN);
 		goto reject;
@@ -695,9 +695,46 @@ static int cb_gtp_ran_info_relay_ind(struct sockaddr_in *peer, union gtpie_membe
 
 	LOGMME(mme, DGTP, LOGL_INFO, "Rx GTP RAN Information Relay\n");
 
+	int rc;
 	unsigned int len = 0;
-	struct msgb *msg = msgb_alloc(4096, "gtpcv1_ran_info");
-	struct bssgp_ran_information_pdu pdu;
+	struct msgb *msg = bssgp_msgb_alloc();
+
+	uint8_t rim_ra_encoded[256];
+	unsigned int rim_ra_encoded_len = 0;
+	struct bssgp_rim_routing_info rim_ra;
+
+	unsigned int rim_ra_discr_encoded_len = 0;
+	uint8_t rim_ra_discr;
+
+	/* Read RIM Routing Address Discriminator (optional) */
+	rc = gtpie_gettlv(ie, GTPIE_RIM_RA_DISCR, 0, &rim_ra_discr_encoded_len, &rim_ra_discr,
+			  sizeof(rim_ra_discr));
+	if (rc || rim_ra_discr_encoded_len <= 0) {
+		LOGMME(mme, DGTP, LOGL_NOTICE, "Rx GTP RAN Information Relay: No RIM Routing Address Discriminator IE found!\n");
+
+		/* It is not an error when the RIM ROUTING ADDRESS DISCRIMINATOR IE is missing. The RIM ROUTING ADDRESS
+		 * DISCRIMINATOR IE is an optional IE. When it is missing, the RIM Routing Address shall be processed
+		 * as an RNC address ("0001")  See also: 3GPP TS 29.060 */
+		rim_ra_discr = BSSGP_RIM_ROUTING_INFO_UTRAN;
+	}
+
+	/* Read RIM Routing Address (optional) */
+	rc = gtpie_gettlv(ie, GTPIE_RIM_ROUT_ADDR, 0, &rim_ra_encoded_len, rim_ra_encoded, sizeof(rim_ra_encoded));
+	if (rc || rim_ra_encoded_len <= 0) {
+		LOGMME(mme, DGTP, LOGL_ERROR, "Rx GTP RAN Information Relay: No RIM Routing Address IE found!\n");
+
+		/* TODO: The (usually included) RIM ROUTING ADDRESS field is an optional field. However, we cannot
+		 * proceed without a destination address. A possible way to fix this would be a default route that
+		 * can be configured via the VTY. */
+		goto ret_error;
+	} else {
+		rc = bssgp_parse_rim_ra(&rim_ra, rim_ra_encoded, rim_ra_encoded_len, rim_ra_discr);
+		if (rc < 0) {
+			LOGMME(mme, DGTP, LOGL_ERROR,
+			       "Rx GTP RAN Information Relay: Failed parsing RIM Routing Address/RIM Routing Address Discriminator IE!\n");
+			goto ret_error;
+		}
+	}
 
 	if (gtpie_gettlv(ie, GTPIE_RAN_T_CONTAIN, 0, &len, msgb_data(msg), 4096) || len <= 0) {
 		LOGMME(mme, DGTP, LOGL_ERROR, "Rx GTP RAN Information Relay: No Transparent Container IE found!\n");
@@ -706,13 +743,8 @@ static int cb_gtp_ran_info_relay_ind(struct sockaddr_in *peer, union gtpie_membe
 	msgb_put(msg, len);
 	msgb_bssgph(msg) = msg->data;
 	msgb_nsei(msg) = 0;
-	if (bssgp_parse_rim_pdu(&pdu, msg) < 0) {
-		LOGMME(mme, DGTP, LOGL_ERROR, "Rx GTP RAN Information Relay: Failed parsing Transparent Container IE!\n");
-		goto ret_error;
-	}
 
-	msgb_free(msg);
-	return sgsn_rim_rx_from_gtp(&pdu);
+	return sgsn_rim_rx_from_gtp(msg, &rim_ra);
 
 ret_error:
 	msgb_free(msg);
